import { EventEmitter, Component, ElementRef, Input, Output, NgModule } from '@angular/core';
import { _, findPlugin, assignGridOptions, assignDeep, setInstance, deleteInstance } from '@grapecity/dataviews.common';
import { DataView } from '@grapecity/dataviews.core';

/**
 * @module Angular
 *
 * This module provides NgModule and component for Angular integration of {@link DataView} control.
 */
function isTrue(val) {
    if (_.isNil(val)) {
        return false;
    }
    if (_.isBoolean(val)) {
        return val !== false;
    }
    if (_.isNumber(val)) {
        return val !== 0;
    }
    return true;
}
function makeGridLayout(options) {
    const Plugin = findPlugin('GridLayout');
    if (!Plugin) {
        throw new Error('@grapecity/dataviews.grid is not loaded');
    }
    return new Plugin(options);
}
const ɵ0 = (v) => v.selectionChanged, ɵ1 = (v) => v.focusChanged, ɵ2 = (v) => v.grouping, ɵ3 = (v) => v.rowClick, ɵ4 = (v) => v.rowDbClick, ɵ5 = (v) => v.columnReorder, ɵ6 = (v) => v.rowReorder, ɵ7 = (v) => v.columnChanged, ɵ8 = (v) => v.editing, ɵ9 = (v) => v.beforeLoadRange;
const eventMap = {
    selectionChanged: ɵ0,
    focusChanged: ɵ1,
    grouping: ɵ2,
    rowClick: ɵ3,
    rowDbClick: ɵ4,
    columnReorder: ɵ5,
    rowReorder: ɵ6,
    columnChanged: ɵ7,
    editing: ɵ8,
    beforeLoadRange: ɵ9,
};
function isEmpty(a) {
    return !a || a.length === 0;
}
class GridEvent extends EventEmitter {
    constructor(name, component) {
        super();
        this.emitter = (event) => {
            this.emit(event);
        };
        this.name = name;
        this.component = component;
    }
    subscribe(generatorOrNext, error, complete) {
        const onObservers = isEmpty(this.observers);
        const sub = super.subscribe(generatorOrNext, error, complete);
        if (onObservers) {
            this.register();
        }
        sub.add(() => {
            if (isEmpty(this.observers)) {
                this.unregister();
            }
        });
        return sub;
    }
    get gridEvent() {
        return eventMap[this.name](this.component.dataView);
    }
    register() {
        if (this.component.dataView && !this.gridEvent.contains(this.emitter)) {
            this.gridEvent.addHandler(this.emitter);
        }
    }
    unregister() {
        if (this.component.dataView) {
            this.gridEvent.removeHandler(this.emitter);
        }
    }
}
/**
 * Angular component for {@link DataView} control
 *
 * See examples:
 * - {@sample Features/Basics/Introduction/angular Basic Data Grid}
 * - {@sample DataViews/Calendar/Calendar/angular Calendar Layout}
 * - {@sample DataViews/Card/CardLayoutEngine/angular Card Layout}
 * - {@sample DataViews/Trellis/TrellisWithGridLayout/angular Trellis Layout}
 * - {@sample DataViews/Masonry/MasonryLayout/angular Masonry Layout}
 * - {@sample DataViews/Timeline/TimelineGrouping/angular Timeline Grouping}
 */
class DataViewComponent {
    /**
     * Initializes new instance of DataViewComponent
     * @param container a reference to host container
     */
    constructor(container) {
        this._initialized = false;
        /**
         * This event occurs on each creation of underlying DataView component.
         */
        this.created = new EventEmitter();
        /**
         * This event occurs when the user changes the selection using UI actions.
         */
        this.selectionChanged = new GridEvent('selectionChanged', this);
        /**
         * This event occurs when the focused row is changed. The event fires on [[GridLayout]] or [[HorizontalLayout]]
         */
        this.focusChanged = new GridEvent('focusChanged', this);
        /**
         * This event occurs when the user updates groups with UI actions. The event is fired before the actual update happens
         */
        this.grouping = new GridEvent('grouping', this);
        /**
         * This event occurs when clicking the row
         */
        this.rowClick = new GridEvent('rowClick', this);
        /**
         * This event occurs when double-clicking the row
         */
        this.rowDbClick = new GridEvent('rowDbClick', this);
        /**
         * This event occurs when the user starts dragging the column and before dropping the column in the column header
         */
        this.columnReorder = new GridEvent('columnReorder', this);
        /**
         * This event occurs when the user starts dragging the row and before inserting the row at new position
         */
        this.rowReorder = new GridEvent('rowReorder', this);
        /**
         * This event occurs when the user changes the column visibility, pinned option, or column width, or expands or collapses the column group from UI
         */
        this.columnChanged = new GridEvent('columnChanged', this);
        /**
         * This event occurs before editing, once editing has started, before editing has ended, or editing has stopped
         */
        this.editing = new GridEvent('editing', this);
        /**
         * This event occurs before loading next data range.
         */
        this.beforeLoadRange = new GridEvent('beforeLoadRange', this);
        this.container = container;
    }
    /**
     * Specifies an array of objects for the grid model or a dataSource object
     */
    get data() {
        return this._data;
    }
    set data(value) {
        if (value === this._data) {
            return;
        }
        this._data = value;
        this.recreateView();
    }
    /**
     * Specifies an array of column definitions to initialize the grid
     */
    get cols() {
        return this._cols;
    }
    set cols(value) {
        if (_.isEqual(value, this._cols)) {
            return;
        }
        this._cols = value;
        if (this._initialized) {
            this.dataView.columns = value;
            this.dataView.invalidate();
        }
    }
    /**
     * Specifies a JSON object used to deserialize the grid from
     */
    get config() {
        return this._config;
    }
    set config(value) {
        if (value === this._config) {
            return;
        }
        this._config = value;
        this.recreateView();
    }
    /**
     * Specifies layout strategy. Default value is instance of {@link GridLayout} with default options
     */
    get layout() {
        return this._layout;
    }
    set layout(value) {
        if (value === this._layout) {
            return;
        }
        this._layout = value;
        this.recreateView();
    }
    /**
     * Specifies options of layout strategy and other common options like filtering/sorting/grouping.
     */
    get options() {
        return this._options;
    }
    set options(value) {
        if (_.isEqual(value, this._options)) {
            return;
        }
        this._options = value;
        if (this.dataView && !_.isEmpty(this._options)) {
            assignGridOptions(this.dataView.options, this._options);
            this.dataView.invalidate();
        }
    }
    /**
     * Specifies options of grouping strategy.
     */
    get groupingOptions() {
        return this._groupingOptions;
    }
    set groupingOptions(value) {
        if (_.isEqual(value, this._groupingOptions)) {
            return;
        }
        this._groupingOptions = value;
        if (this.dataView && !_.isEmpty(this._groupingOptions)) {
            assignDeep(this.dataView.layoutEngine.groupStrategy.options, this._groupingOptions);
            this.dataView.invalidate();
        }
    }
    ngAfterViewInit() {
        this.createView();
        this._initialized = true;
        if (isTrue(this.autoFocus)) {
            this.container.nativeElement.focus();
        }
    }
    ngOnDestroy() {
        this.destroyView();
        this._initialized = false;
    }
    createView() {
        if (!this._layout) {
            this._layout = makeGridLayout(this._options || {});
        }
        this.dataView = new DataView(this.container.nativeElement, this.data, this.cols || this.config, this._layout);
        this.registerHandlers();
        setInstance(this.dataView.uid, this.dataView);
        this.created.emit(this.dataView);
        this.applyOptions();
    }
    destroyView() {
        if (this.dataView) {
            deleteInstance(this.dataView.uid);
            this.unregisterHandlers();
            this.dataView.destroy();
            this.dataView = undefined;
        }
    }
    recreateView() {
        if (!this._initialized) {
            return;
        }
        this.destroyView();
        this.createView();
    }
    registerHandlers() {
        for (const eventName of Object.keys(eventMap)) {
            this[eventName].register();
        }
    }
    unregisterHandlers() {
        for (const eventName of Object.keys(eventMap)) {
            this[eventName].unregister();
        }
    }
    applyOptions(forceInvalidate = false) {
        let hasOptions = false;
        if (!_.isEmpty(this._options)) {
            hasOptions = true;
            assignGridOptions(this.dataView.options, this._options);
        }
        if (!_.isEmpty(this._groupingOptions) && this.dataView.layoutEngine.groupStrategy) {
            hasOptions = true;
            assignDeep(this.dataView.layoutEngine.groupStrategy.options, this._groupingOptions);
        }
        if (forceInvalidate || hasOptions) {
            this.dataView.invalidate();
        }
    }
}
DataViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-dataview',
                template: `<ng-content></ng-content>`,
                styles: [':host { display: block; width: 100%; height: 100% }']
            },] }
];
DataViewComponent.ctorParameters = () => [
    { type: ElementRef }
];
DataViewComponent.propDecorators = {
    data: [{ type: Input }],
    cols: [{ type: Input }],
    config: [{ type: Input }],
    layout: [{ type: Input }],
    options: [{ type: Input }],
    groupingOptions: [{ type: Input }],
    autoFocus: [{ type: Input }],
    created: [{ type: Output }],
    selectionChanged: [{ type: Output }],
    focusChanged: [{ type: Output }],
    grouping: [{ type: Output }],
    rowClick: [{ type: Output }],
    rowDbClick: [{ type: Output }],
    columnReorder: [{ type: Output }],
    rowReorder: [{ type: Output }],
    columnChanged: [{ type: Output }],
    editing: [{ type: Output }],
    beforeLoadRange: [{ type: Output }]
};
/**
 * Angular module with {@link DataViewComponent} included for importing in an angular application
 */
// tslint:disable-next-line:no-unnecessary-class
class DataViewModule {
}
DataViewModule.decorators = [
    { type: NgModule, args: [{
                declarations: [DataViewComponent],
                exports: [DataViewComponent],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { DataViewComponent, DataViewModule, ɵ0, ɵ1, ɵ2, ɵ3, ɵ4, ɵ5, ɵ6, ɵ7, ɵ8, ɵ9 };
//# sourceMappingURL=grapecity-dataviews.angular10.js.map
